/* message.h
 * Neil Moore <neil@cs.uky.edu>
 * 19 Apr 2001
 *
 * Part of parstab.  See COPYRIGHT for copyright information.
 */

#ifndef _MESSAGE_H_INCLUDED
#define _MESSAGE_H_INCLUDED

#include <lp.h>
#include "misc.h"
#include "timing.h"

#define MSG_STACK      1
#define MSG_DIE        2
#define MSG_WAITING    3
#define MSG_MODEL      4
#define MSG_NAKWORK    5
#define MSG_SENTJOB    6
#define MSG_DIED       7
#define MSG_PROG       8
#define MSG_STACK_UPD  9
#define MSG_MODEL_UPD  10
#define MSG_MAX        10

#define MTAG_MSG       1
#define MTAG_NOTIFY    2

extern char *MessageNames[];

struct message {
	machine_t from;
	int type;
	int misc;
	int misc2;
	int size;
	void *data;
};

int send_stack(machine_t machine, int top, int *stack, int *aux);
int send_stack_upd(machine_t machine, int top, int *stack, int *aux);
int send_die(machine_t machine);
int send_waiting(machine_t machine);
int send_model(machine_t machine, struct lp *prog, int *model);
int send_model_upd(machine_t machine, struct lp *prog, int *model);
int send_nakwork(machine_t machine);
int send_sentjob(machine_t machine, machine_t to_whom);
int send_died(machine_t machine);
int send_prog(machine_t machine, struct lp *prog, int cldnum, int children);

int wait_for_message(struct message *m);
int parse_message(struct message *msg);
int notify_message(struct message *msg);

void not_expecting(int type);
char *message_name(int type);

#endif /* _MESSAGE_H_INCLUDED */
