/* misc.c
 * Neil Moore <neil@cs.uky.edu>
 * 15 May 2000
 *
 * Part of parstab.  See COPYRIGHT for copyright information.
 *
 */

#include <stdlib.h>
#include <lp_util.h>
#include "misc.h"

machine_t ml_pop(struct mac_list **l)
{
	machine_t result;
	struct mac_list *old = *l;
	*l = old->next;
	result = old->machine;
	free(old);

	return result;
}

void ml_add(struct mac_list **l, machine_t machine) 
{
	struct mac_list *curr;
	
	lp_assert(l != NULL);
	
	curr = lp_malloc(sizeof(struct mac_list));
	curr->next = *l;
	curr->machine = machine;
	*l = curr;
}

void ml_free(struct mac_list *l) 
{
	struct mac_list *old;
	while(l != NULL) {
		old = l;
		l = l->next;
		free(old);
	}
}


struct mac_queue *mq_new(void) 
{
	return lp_calloc(1,sizeof(struct mac_queue));
}


void mq_free(struct mac_queue *mq) 
{
	if(mq->head)
		ml_free(mq->head);
	free(mq);
}


void mq_enqueue(struct mac_queue *mq, machine_t m) 
{
	struct mac_list *curr = lp_malloc(sizeof(struct mac_list));
	curr->next = NULL;
	curr->machine = m;

	
	if(mq->tail == NULL) {
		mq->tail = curr;
	} else {
		mq->tail->next = curr;
		mq->tail = curr;
	}
	
		
	if(mq->head == NULL)
		mq->head = curr;
}


machine_t mq_head(const struct mac_queue *mq) 
{
	if(mq->head == NULL)
		return 0;
	return mq->head->machine;
}


machine_t mq_dequeue(struct mac_queue *mq) 
{
	machine_t m;
	struct mac_list *old = mq->head;
	
	if(old == NULL)
		return 0;
	
	m = old->machine;

	mq->head = old->next;
	free(old);
	
	if(mq->head == NULL)
		mq->tail = NULL;
	
	return m;
}


int mq_empty(const struct mac_queue *mq) 
{
	return (mq->head == NULL);
}

