/* misc.h
 * Neil Moore <neil@cs.uky.edu>
 * 22 May 2001
 *
 * Part of parstab.  See COPYRIGHT for copyright information.
 *
 */

#ifndef __MISC_H_INCLUDED
#define __MISC_H_INCLUDED

#include <stdio.h>
#include "timing.h"

typedef int machine_t;

/* The FILE to do logging to.  All logging is done in append mode.
 * FILE is actually defined in master.c */
extern FILE *logfile;

/* Log level, and toggle for verbose timing.  Defined in master.c */
extern int Logging;
extern int Verbose_timing;

/* Macros to print to logfile.  Prints a line with the format:
 *   [tid] message
 * DO_LOG logs a single string; DO_LOGn treat the string as printf
 * format strings; n is one plus the number of additional arguments.
 * So, for example, one might use:
 *   DO_LOG3("ERROR: %d (%s)", errno, strerror(errno));
 * DO_LOG* may only be used as statements.  A trailing semicolon is
 * required
 */
#define DO_LOG(N,msg) do { \
			if(Logging >= N) { \
				begin_time(TIM_LOG, "log"); \
				fprintf(logfile, "[%x] %s\n", pvm_mytid(), (msg)); \
				fflush(logfile); \
				end_time(TIM_LOG, "log"); \
			} \
		    } while(0)
#define DO_LOG2(N,msg,arg) do { \
			if(Logging >= N) { \
				begin_time(TIM_LOG, "log"); \
				fprintf(logfile, "[%x] " msg "\n", pvm_mytid(), (arg)); \
				fflush(logfile); \
				end_time(TIM_LOG, "log"); \
			} \
		    } while(0)
#define DO_LOG3(N,msg,arg1,arg2) do { \
			if(Logging >= N) { \
				begin_time(TIM_LOG, "log"); \
				fprintf(logfile, "[%x] " msg "\n", pvm_mytid(), (arg1), (arg2)); \
				fflush(logfile); \
				end_time(TIM_LOG, "log"); \
			} \
		    } while(0)
#define DO_LOG4(N,msg,arg1,arg2,arg3) do { \
			if(Logging >= N) { \
				begin_time(TIM_LOG, "log"); \
				fprintf(logfile, "[%x] " msg "\n", pvm_mytid(), (arg1), (arg2), (arg3)); \
				fflush(logfile); \
				end_time(TIM_LOG, "log"); \
			} \
		    } while(0)

#define LP_LOG_MODEL  0
#define LP_LOG_NONE   0
#define LP_LOG_ERROR  1
#define LP_LOG_INFO   2
#define LP_LOG_DEBUG  3
#define LP_LOG_EXTRA  4
#define LP_LOG_MAX    4

struct mac_list {
	machine_t machine;
	struct mac_list *next;
};

struct mac_queue {
	struct mac_list *head;
	struct mac_list *tail;
};


machine_t ml_pop(struct mac_list **l);
void ml_add(struct mac_list **l, machine_t machine);
void ml_free(struct mac_list *l);

struct mac_queue *mq_new(void);
void mq_free(struct mac_queue *aq);

void mq_enqueue(struct mac_queue *mq, machine_t m);
machine_t mq_head(const struct mac_queue *mq);
machine_t mq_dequeue(struct mac_queue *mq);

int mq_empty(const struct mac_queue *mq);

#endif /* __MISC_H_INCLUDED */
